@@ -2,7 +2,11 @@
 available here as well to keep the number of imports to a minimum for most
 applications.
 """
-__version__ = '2.4.2'
+from __future__ import division
+from __future__ import absolute_import
+from __future__ import unicode_literals
+from __future__ import print_function
+__version__ = '3.0.0'
 
 from .base import SquirroClient
 from .exceptions import *
