@@ -1,16 +1,30 @@
-from .bulk import BulkApiMixin
-from .exceptions import InputDataError, AuthenticationError, \
-    AuthorizationError, NotFoundError, TransportError, ConnectionError, \
-    UnknownError, ClientError, ConflictError, RetryError
-from .topic import TopicApiMixin
-from .user import UserApiMixin
+from __future__ import division
+from __future__ import absolute_import
+from __future__ import print_function
+
 import json
 import logging
 import os.path
 import sys
+import warnings
+from builtins import range, str
 
+from .exceptions import (
+    AuthenticationError, AuthorizationError, ClientError, ConflictError,
+    ConnectionError, InputDataError, NotFoundError, RetryError, TransportError,
+    UnknownError)
+from .topic import TopicApiMixin
+from .user import UserApiMixin
 
 log = logging.getLogger(__name__)
+# SQ-7983: lower the log level of retry module to give some feedback
+# see bug report for details
+logging.getLogger('urllib3.util.retry').setLevel(logging.DEBUG)
+
+warnings.filterwarnings('ignore', category=ImportWarning)
+# Disable `ResourceWarning` for python 3 due to unclosed request sessions
+if sys.version_info[0] >= 3:
+    warnings.filterwarnings('ignore', category=ResourceWarning)  # noqa
 
 
 #: Default headers for all requests.
@@ -20,7 +34,6 @@ HEADERS = {'Accept': 'application/json'}
 ENDPOINT = {
     'user': 'https://user-api.squirro.com',
     'topic': 'https://topic-api.squirro.com',
-    'bulk': 'https://bulk-api.squirro.com',
 }
 
 #: Version of the Squirro API to be used.
@@ -45,7 +58,7 @@ RETRY = {
 }
 
 
-class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
+class SquirroClient(UserApiMixin, TopicApiMixin):
     """Client to access the Squirro API.
 
     :param client_id: The client id for accessing the API.
@@ -53,7 +66,6 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
     :param cluster: The cluster endpoint for accessing the API.
     :param user_api_url: Endpoint URL for the user API service.
     :param topic_api_url: Endpoint URL for the topic API service.
-    :param bulk_api_url: Endpoint URL for the topic API service.
     :param tenant: Tenant domain.
     :param version: API version to use, defaults to 'v0'.
     :param requests: `requests` instance for HTTP calls.
@@ -67,14 +79,18 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
     :param retry_status_forcelist: A set of integer HTTP status codes that we
         should force a retry on.
     :param retry_backoff_factor: A backoff factor to apply between attempts.
-    :param timeout_secs: How many seconds to wait for data before giving up (default 55).
+    :param timeout_secs: How many seconds to wait for data before giving up
+        (default 55).
 
     Example::
 
         >>> from squirro_client import SquirroClient
         >>> client = SquirroClient('client_id', 'client_secret')
-        >>> client = SquirroClient(None, None, cluster='http://squirro.example.com')
-        >>> client = SquirroClient(None, None, cluster='http://squirro.example.com', retry_total=10)
+        >>> client = SquirroClient(None, None,
+        ...                        cluster='http://squirro.example.com')
+        >>> client = SquirroClient(None, None,
+        ...                        cluster='http://squirro.example.com',
+        ...                        retry_total=10)
     """
 
     def __init__(self, client_id, client_secret, **kwargs):
@@ -94,23 +110,20 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
         if cluster:
             cluster = cluster.rstrip('/')
             base = '%(cluster)s/api/%(api)s'
-            self.user_api_url = base % ({'cluster': cluster, 'api': 'user'})
-            self.topic_api_url = base % ({'cluster': cluster, 'api': 'topic'})
-            self.bulk_api_url = base % ({'cluster': cluster, 'api': 'bulk'})
+            self.user_api_url = base % {'cluster': cluster, 'api': 'user'}
+            self.topic_api_url = base % {'cluster': cluster, 'api': 'topic'}
 
         else:
             user_api_url = kwargs.get('user_api_url') or ENDPOINT['user']
             topic_api_url = kwargs.get('topic_api_url') or ENDPOINT['topic']
-            bulk_api_url = kwargs.get('bulk_api_url') or ENDPOINT['bulk']
 
             self.user_api_url = user_api_url.rstrip('/')
             self.topic_api_url = topic_api_url.rstrip('/')
-            self.bulk_api_url = bulk_api_url.rstrip('/')
 
         # retry default options, use values provided by the caller to update
         # the default retry options
         self.retry = RETRY
-        for k, v in kwargs.iteritems():
+        for k, v in kwargs.items():
             if k.startswith('retry_'):
                 self.retry[k.replace('retry_', '')] = v
 
@@ -169,13 +182,17 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
         :param user_information: Additional information about the user for
             internal use.
 
-        :raises: :class:`squirro_client.exceptions.AuthenticationError` if authentication fails.
+        :raises: :class:`squirro_client.exceptions.AuthenticationError` if
+            authentication fails.
 
         Example::
 
-            >>> client.authenticate(username='test@test.com', password='test')
-            >>> client.authenticate(access_token='token01', refresh_token='token02')
-            >>> client.authenticate(auth_service='salesforce', auth_user='sfdc01')
+            >>> client.authenticate(username='test@test.com',
+            ...                     password='test')
+            >>> client.authenticate(access_token='token01',
+            ...                     refresh_token='token02')
+            >>> client.authenticate(auth_service='salesforce',
+            ...                     auth_user='sfdc01')
 
         """
 
@@ -185,13 +202,13 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
 
         # assemble base request data
         base = {}
-        if not self.client_id is None:
+        if self.client_id is not None:
             base['client_id'] = self.client_id
-        if not self.client_secret is None:
+        if self.client_secret is not None:
             base['client_secret'] = self.client_secret
-        if not self.tenant is None:
+        if self.tenant is not None:
             base['tenant'] = self.tenant
-        if not user_information is None:
+        if user_information is not None:
             base['user_information'] = json.dumps(user_information)
 
         # walk through all possible authentication scenarios
@@ -239,7 +256,7 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
         """
 
         # create endpoint url to fetch new tokens
-        url = '%(endpoint)s/oauth2/token' % ({'endpoint': self.user_api_url})
+        url = '%(endpoint)s/oauth2/token' % {'endpoint': self.user_api_url}
 
         session = self._new_session()
 
@@ -257,7 +274,7 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
             if r.content:
                 res = json.loads(r.content)
         except ValueError as ex:
-            log.debug('unable to decode JSON: %r', ex)
+            log.error('unable to decode JSON: %r', r.content)
             raise TransportError(r.status_code, ex)
 
         # perform error handling for known cases
@@ -298,8 +315,8 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
         # construct headers to be used for all requests
         headers = dict(HEADERS)
         if access_token is not None:
-            authorization = 'Bearer %(token)s' % ({'token': self.access_token})
-            headers['Authorization'] = authorization.encode('utf-8')
+            authorization = 'Bearer %(token)s' % {'token': self.access_token}
+            headers['Authorization'] = authorization
 
         session = self._requests.Session()
         session.headers.update(headers)
@@ -337,7 +354,7 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
 
             try:
                 res = func(
-                    url.encode('utf-8'), timeout=self.timeout_secs, **kwargs)
+                    url, timeout=self.timeout_secs, **kwargs)
             except self._requests.exceptions.RetryError as ex:
                 raise RetryError(None, ex[0])
 
@@ -387,13 +404,12 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
                     ret = True
                     break
 
-                log.info('Access token %r not refreshed on behalf of ' + \
+                log.info('Access token %r not refreshed on behalf of '
                          'refresh token %r. Retry number %d',
                          self.access_token, self.refresh_token, try_count)
 
         except Exception as ex:
-            log.debug('unable to refresh access token: %r', ex)
-
+            log.exception('unable to refresh access token: %r', ex)
 
         return ret
 
@@ -412,7 +428,7 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
             if res.content:
                 data = json.loads(res.content)
         except ValueError as ex:
-            log.debug('unable to decode JSON: %r', ex)
+            log.error('unable to decode JSON: %r', res.content)
             raise TransportError(None, ex)
 
         # if the status code matches to what is expected we return the data
@@ -424,7 +440,7 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
         if res.status_code == 400:
             raise ClientError(res.status_code, error, data)
 
-        elif res.status_code == 401:
+        elif res.status_code in [401, 403]:
             raise AuthorizationError(res.status_code, error)
 
         elif res.status_code == 404:
@@ -466,8 +482,8 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
         if not hasattr(sys, "frozen"):
             # The `frozen` attribute is set in py2exe
             return None
-        exe_dir = os.path.dirname(unicode(sys.executable,
-                                          sys.getfilesystemencoding()))
+        exe_dir = os.path.dirname(str(sys.executable,
+                                      sys.getfilesystemencoding()))
         ca_bundle = os.path.join(exe_dir, 'cacert', 'cacert.pem')
         if not os.path.exists(ca_bundle):
             return False
