@@ -1,9 +1,18 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+from __future__ import print_function
+from __future__ import division
+
 import base64
 import hashlib
 import logging
 import os
+from builtins import object
+
+from past.utils import old_div
 
 from .item_uploader import ItemUploader
+from .util import deprecation
 
 log = logging.getLogger(__name__)
 
@@ -35,7 +44,9 @@ class DocumentUploader(object):
 
         >>> from squirro_client import DocumentUploader
         >>> import os
-        >>> uploader = DocumentUploader(project_title='My Project', token='<your token>', cluster='https://next.squirro.net/')
+        >>> uploader = DocumentUploader(
+        ...     project_title='My Project', token='<your token>',
+        ...     cluster='https://demo.squirro.net/')
         >>> uploader.upload(os.path.expanduser('~/Documents/test.pdf'))
         >>> uploader.flush()
 
@@ -44,14 +55,16 @@ class DocumentUploader(object):
     * By default (i.e. for all document mime-types) map the original document
       size to a keyword field named "Doc Size":
 
-        >>> mapping = {'default': {'sq:size_orig': 'Doc Size', 'sq:content-mime-type': 'Mime Type'}}
+        >>> mapping = {'default': {'sq:size_orig': 'Doc Size',
+        ...                        'sq:content-mime-type': 'Mime Type'}}
         >>> uploader = DocumentUploader(metadata_mapping=mapping)
 
     * For a specific mime-type (i.e.
       'application/vnd.oasis.opendocument.text') map the "meta:word-count"
       meta-data filed value to a keyword field named "Word Count":
 
-        >>> mapping = {'application/vnd.oasis.opendocument.text': {'meta:word-count': 'Word Count'}}
+        >>> mapping = {'application/vnd.oasis.opendocument.text': {
+        ...     'meta:word-count': 'Word Count'}}
         >>> uploader = DocumentUploader(metadata_mapping=mapping)
 
     Default meta-data fields available for mapping usage:
@@ -67,42 +80,100 @@ class DocumentUploader(object):
                  default_mime_type_keyword=True,
                  timeout_secs=TIMEOUT_SECS, **kwargs):
 
-        if metadata_mapping is None:
-            metadata_mapping = {}
+        # assemble steps_config configuration
+        source_id = kwargs.pop('source_id', None)
+        pipeline_workflow_id = kwargs.pop('pipeline_workflow_id', None)
+        pipeline_workflow_name = kwargs.pop('pipeline_workflow_name', None)
+        steps_config = kwargs.pop('steps_config', None)
 
-        # assemble processing configuration
-        proc_config = {
-            'content-conversion': {
-                'enabled': True,
-                'metadata-mapping': {
-                    'default': {}
-                }
-            }
-        }
-        mapping = proc_config['content-conversion']['metadata-mapping']
-        mapping.update(metadata_mapping)
-
-        if default_mime_type_keyword and \
-                'sq:content-mime-type' not in mapping['default']:
-            mapping['default']['sq:content-mime-type'] = 'MIME Type'
-
-        # check for an explicit processing config
+        # deprecation warning for an explicit processing config
         if 'processing_config' in kwargs:
-            pc = kwargs.get('processing_config')
-            if pc:
-                proc_config.update(pc)
-            del kwargs['processing_config']
+            msg = "processing_config is deprecated and is not supported" \
+                "anymore. Please use pipeline_workflow_id or " \
+                "pipeline_workflow_name instead."
+            deprecation(msg)
+            raise NotImplementedError(msg)
 
         # disable bulk indexing as document conversion is not supported
+        # For compatibility.
         if 'bulk_index' in kwargs:
-            del kwargs['bulk_index']
+            msg = 'bulk_index is supported by the new ingester automatically ' \
+                'and can not be specified from outside anymore. Please ' \
+                'remove this parameter from the class constructor'
+            deprecation(msg)
+            raise NotImplementedError(msg)
 
-        # item uploader
+        kwargs['batch_size'] = batch_size
+        kwargs['timeout_secs'] = timeout_secs
         self.batch_size = batch_size
         self.batch_size_mb = batch_size_mb
-        self.uploader = ItemUploader(
-            batch_size=batch_size, processing_config=proc_config,
-            timeout_secs=timeout_secs, **kwargs)
+
+        # Initialize priority
+        mapping = metadata_mapping or {}
+        if (default_mime_type_keyword and
+                'sq:content-mime-type' not in mapping.get('default', {})):
+            mapping.setdefault('default', {})['sq:content-mime-type'] = \
+                'MIME Type'
+        default_steps_config = [
+            {
+                "id": "deduplication",
+                "name": "Deduplication",
+                "type": "deduplication",
+                "config": {"policy": "replace"}
+            },
+            {
+                "id": "content-augmentation",
+                "name": "Content Augmentation",
+                "type": "content-augmentation"
+            },
+            {
+                "id": "content-conversion",
+                "name": "Content Extraction",
+                "type": "content-conversion",
+                "metadata-mapping": mapping
+            },
+            {
+                "id": "language-detection",
+                "name": "Language Detection",
+                "type": "language-detection"
+            },
+            {
+                "id": "cleanup",
+                "name": "Content Standardization",
+                "type": "cleanup"
+            },
+            {
+                "id": "index",
+                "name": "Indexing",
+                "type": "index"
+            },
+            {
+                "id": "cache",
+                "name": "Cache Cleaning",
+                "type": "cache"
+            }
+        ]
+
+        if source_id:
+            # Use existing source
+            self.uploader = ItemUploader(source_id=source_id, **kwargs)
+
+        elif pipeline_workflow_id:
+            # Use existing pipeline workflow
+            self.uploader = ItemUploader(
+                pipeline_workflow_id=pipeline_workflow_id, **kwargs)
+
+        elif not steps_config:
+            # Create pipeline workflow with default steps config
+            self.uploader = ItemUploader(
+                pipeline_workflow_name=pipeline_workflow_name,
+                steps_config=default_steps_config, **kwargs)
+
+        else:
+            # Create pipeline workflow with given steps config
+            self.uploader = ItemUploader(
+                pipeline_workflow_name=pipeline_workflow_name,
+                steps_config=steps_config, **kwargs)
 
         # internal state
         self._items = []
@@ -110,7 +181,8 @@ class DocumentUploader(object):
 
     def upload(self, filename, mime_type=None, title=None, doc_id=None,
                keywords=None, link=None, created_at=None,
-               filename_encoding=None, content_url=None):
+               filename_encoding=None, content_url=None,
+               priority=0, pipeline_workflow_id=None):
         """Method which will use the provided ``filename`` to create a Squirro
         item for upload. Items are buffered internally and uploaded according
         to the specified batch size. If `mime_type` is not provided a simple
@@ -127,6 +199,13 @@ class DocumentUploader(object):
         :param filename_encoding: Encoding of the filename.
         :param content_url: Storage URL of this file. If this is set, the
             Squirro cluster will not copy the file.
+        :param priority: int, describing the priority of ingestion for the
+            dataset to be loaded. Currently only supports a value of `0` or `1`.
+            `0` means that the items are loaded in an asynchronous fashion and
+            `1` would mean that the items are loaded in a synchronous fashion.
+        :param pipeline_workflow_id: str, id of an existing pipeline
+            workflow which should be used to process the current batch of items.
+            Can only be used with parameter `priority` set to 1.
 
         Example:
 
@@ -134,12 +213,17 @@ class DocumentUploader(object):
         >>> mime_type = 'application/pdf'
         >>> title = 'My Test Document'
         >>> doc_id = 'doc01'
-        >>> keywords = {'Author': ['John Smith'], 'Tags': ['sales', 'marketing']}
+        >>> keywords = {'Author': ['John Smith'], 'Tags': ['sales',
+        ...                                                'marketing']}
         >>> link = 'http://example.com/test.pdf'
         >>> created_at = '2014-07-10T21:26:15'
-        >>> uploader.upload(filename, mime_type, title, doc_id, keywords, link, created_at)
+        >>> uploader.upload(filename, mime_type, title, doc_id, keywords,
+        ...                 link, created_at)
         """
 
+        self.priority = priority
+        self.adhoc_pipeline_workflow_id = pipeline_workflow_id
+
         item_id = doc_id
         content_size = 0
         content = None
@@ -148,7 +232,7 @@ class DocumentUploader(object):
             # read the raw file content and encode it
             raw = open(filename, 'rb').read()
             content = base64.b64encode(raw)
-            content_size = len(content) / 1024 / 1024
+            content_size = old_div(len(content), 1024 / 1024)
 
             # build a checksum over the original file content and use it as the
             # item identifier
@@ -163,7 +247,8 @@ class DocumentUploader(object):
         if self._items_size_mb + content_size >= self.batch_size_mb:
             self.flush()
 
-        if isinstance(filename, unicode):
+        # TODO: python3 test this
+        if isinstance(filename, str):
             decoded_name = filename.encode('utf8')
         elif filename_encoding:
             decoded_name = filename.decode(filename_encoding).encode('utf8')
@@ -206,12 +291,22 @@ class DocumentUploader(object):
         self._items.append(item)
         self._items_size_mb += content_size
 
+        if self.priority == 1:
+            self.flush()
+
         if len(self._items) >= self.batch_size:
             self.flush()
 
     def flush(self):
         """Flush the internal buffer by uploading all documents."""
         if self._items:
-            self.uploader.upload(self._items)
+            if self.priority:
+                self.uploader.upload(
+                    self._items, priority=self.priority,
+                    pipeline_workflow_id=self.adhoc_pipeline_workflow_id)
+                self.adhoc_pipeline_workflow_id = None
+                self.priority = 0
+            else:
+                self.uploader.upload(self._items)
             self._items = []
             self._items_size_mb = 0
