@@ -1,5 +1,11 @@
-# encoding: utf-8
+# -*- coding: utf-8 -*-
+from __future__ import division
+from __future__ import absolute_import
+from __future__ import print_function
+
 import json
+import re
+from builtins import object
 
 
 class UserApiMixin(object):
@@ -16,6 +22,7 @@ class UserApiMixin(object):
         """Return data about a specific user, including any custom values.
 
         :param user_id: User identifier.
+        :param api_version: The version of the API to use (optional).
         :returns: A dictionary which contains the user data.
 
         Example::
@@ -27,9 +34,9 @@ class UserApiMixin(object):
 
         """
 
-        url = '%(endpoint)s/%(version)s/%(tenant)s/users/%(user)s' % ({
+        url = '%(endpoint)s/%(version)s/%(tenant)s/users/%(user)s' % {
             'endpoint': self.user_api_url, 'version': api_version,
-            'tenant': self.tenant, 'user': user_id})
+            'tenant': self.tenant, 'user': user_id}
         res = self._perform_request('get', url)
         return self._process_response(res)
 
@@ -44,17 +51,18 @@ class UserApiMixin(object):
 
         Example::
 
-            >>> client.update_user_data('H5Qv-WhgSBGW0WL8xolSCQ', {'name': 'Alexander Sennhauser'})
+            >>> client.update_user_data('H5Qv-WhgSBGW0WL8xolSCQ',
+            ...                         {'name': 'John Smith'})
             {u'email': u'test@test.com',
              u'id': u'H5Qv-WhgSBGW0WL8xolSCQ',
-             u'name': u'Alexander Sennhauser',
+             u'name': u'John Smith',
              u'tenant': u'test'}
 
         """
 
-        url = '%(endpoint)s/%(version)s/%(tenant)s/users/%(user)s' % ({
+        url = '%(endpoint)s/%(version)s/%(tenant)s/users/%(user)s' % {
             'endpoint': self.user_api_url, 'version': api_version,
-            'tenant': self.tenant, 'user': user_id})
+            'tenant': self.tenant, 'user': user_id}
 
         if api_version == 'v1':
             headers = {'Content-Type': 'application/json'}
@@ -85,10 +93,12 @@ class UserApiMixin(object):
              u'state': u'ok'}
         """
 
-        url = '%(ep)s/%(version)s/%(tenant)s' \
-              '/authentications/%(service)s/%(user_id)s' % ({
-                  'ep': self.user_api_url, 'version': self.version,
-                  'tenant': self.tenant, 'service': service, 'user_id': user_id})
+        url = ('%(ep)s/%(version)s/%(tenant)s'
+               '/authentications/%(service)s/%(user_id)s') % {
+                   'ep': self.user_api_url, 'version': self.version,
+                   'tenant': self.tenant, 'service': service,
+                   'user_id': user_id
+               }
 
         res = self._perform_request('get', url)
         return self._process_response(res, [200])
@@ -113,7 +123,11 @@ class UserApiMixin(object):
 
         Example::
 
-            >>> client.new_or_modify_authentication('H5Qv-WhgSBGW0WL8xolSCQ', 'facebook', '1234567', 'token', 'secret', 'John Smith', datetime.datetime(2014, 1, 9, 8, 40), datetime.datetime(2014, 1, 9, 8, 40))
+            >>> client.new_or_modify_authentication(
+            ...     'H5Qv-WhgSBGW0WL8xolSCQ', 'facebook', '1234567', 'token',
+            ...     'secret', 'John Smith',
+            ...     datetime.datetime(2014, 1, 9, 8, 40),
+            ...     datetime.datetime(2014, 1, 9, 8, 40))
             {u'access_secret': u'secret',
              u'access_secret_expires': u'2014-01-09T08:40:00',
              u'access_token': u'token',
@@ -123,23 +137,24 @@ class UserApiMixin(object):
              u'state': u'ok'}
         """
 
-        url = '%(ep)s/%(version)s/%(tenant)s' \
-              '/authentications/%(service)s/%(user_id)s' % ({
-                  'ep': self.user_api_url, 'version': self.version,
-                  'tenant': self.tenant, 'service': service, 'user_id': user_id})
+        url = ('%(ep)s/%(version)s/%(tenant)s'
+               '/authentications/%(service)s/%(user_id)s') % {
+                   'ep': self.user_api_url, 'version': self.version,
+                   'tenant': self.tenant, 'service': service, 'user_id': user_id
+               }
 
         # build data
         data = {
             'service_user': service_user, 'access_token': access_token
         }
-        if not display_name is None:
+        if display_name is not None:
             data['display_name'] = display_name
-        if not access_secret is None:
+        if access_secret is not None:
             data['access_secret'] = access_secret
-        if not access_token_expires is None:
+        if access_token_expires is not None:
             data['access_token_expires'] = self._format_date(
                 access_token_expires)
-        if not access_secret_expires is None:
+        if access_secret_expires is not None:
             data['access_secret_expires'] = self._format_date(
                 access_secret_expires)
 
@@ -163,13 +178,16 @@ class UserApiMixin(object):
 
         Example::
 
-            >>> client.delete_authentication('H5Qv-WhgSBGW0WL8xolSCQ', 'facebook')
+            >>> client.delete_authentication('H5Qv-WhgSBGW0WL8xolSCQ',
+            ...                              'facebook')
         """
 
-        url = '%(ep)s/%(version)s/%(tenant)s' \
-              '/authentications/%(service)s/%(user_id)s' % ({
-                  'ep': self.user_api_url, 'version': self.version,
-                  'tenant': self.tenant, 'service': service, 'user_id': user_id})
+        url = ('%(ep)s/%(version)s/%(tenant)s'
+               '/authentications/%(service)s/%(user_id)s') % {
+                   'ep': self.user_api_url, 'version': self.version,
+                   'tenant': self.tenant, 'service': service,
+                   'user_id': user_id
+               }
 
         res = self._perform_request('delete', url)
         return self._process_response(res, [204])
@@ -182,19 +200,22 @@ class UserApiMixin(object):
         :param type: Token type, either `user` or `service`.
         :param for_client_id: Client id of the service the token is for. Only
             used when type is `service`.
-        :param project_permissions: List of permissions granted with this grant.
-        :returns: A dictionary containing the `refresh_token` and the `id` of the
+        :param project_permissions: List of permissions granted with this
             grant.
+        :returns: A dictionary containing the `refresh_token` and the `id` of
+            the grant.
 
         Example::
 
-            >>> client.new_grant('H5Qv-WhgSBGW0WL8xolSCQ', 'service', 'abdc0183029498f20c38')
-            {u'id': u'yL0dnd17RNe9Jpn7uAa8Bw', u'refresh_token': u'b4c7440364afb80c2dd72fec18a506e57d7b0450fc0c8f10d3647b8b549587d9a838fe841950e625e7f2e3599a071a3c166936b159a314e2d538b0b773894ea2'}
+            >>> client.new_grant('H5Qv-WhgSBGW0WL8xolSCQ', 'service',
+            ...                  'abdc0183029498f20c38')
+            {u'id': u'yL0dnd17RNe9Jpn7uAa8Bw',
+             u'refresh_token': u'b4c7440364afb80c2...14e2d538b0b773894ea2'}
         """
 
-        url = '%(ep)s/%(version)s/%(tenant)s/users/%(user)s/grants' % ({
+        url = '%(ep)s/%(version)s/%(tenant)s/users/%(user)s/grants' % {
             'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'user': user_id})
+            'tenant': self.tenant, 'user': user_id}
 
         params = {
             'type': type, 'client_id': self.client_id,
@@ -221,7 +242,8 @@ class UserApiMixin(object):
 
         Example::
 
-            >>> client.delete_grant('H5Qv-WhgSBGW0WL8xolSCQ', 'yL0dnd17RNe9Jpn7uAa8Bw')
+            >>> client.delete_grant(
+            ...     'H5Qv-WhgSBGW0WL8xolSCQ', 'yL0dnd17RNe9Jpn7uAa8Bw')
         """
 
         return self.delete_session(user_id, grant_id)
@@ -236,12 +258,14 @@ class UserApiMixin(object):
         Example::
 
             >>> client.get_user_grants('H5Qv-WhgSBGW0WL8xolSCQ')
-            {u'grants': [{u'created_at': u'2012-12-21 16:00:01', u'valid_to': u'2022-12-21 16:00:01', u'id': u'yL0dnd17RNe9Jpn7uAa8Bw', u'type': u'user'}]}
+            {u'grants': [{u'created_at': u'2012-12-21 16:00:01',
+                          u'valid_to': u'2022-12-21 16:00:01',
+                          u'id': u'yL0dnd17RNe9Jpn7uAa8Bw', u'type': u'user'}]}
         """
 
-        url = '%(ep)s/%(version)s/%(tenant)s/users/%(user)s/grants' % ({
+        url = '%(ep)s/%(version)s/%(tenant)s/users/%(user)s/grants' % {
             'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'user': user_id})
+            'tenant': self.tenant, 'user': user_id}
 
         params = {
             'client_id': self.client_id, 'client_secret': self.client_secret
@@ -258,20 +282,26 @@ class UserApiMixin(object):
 
         Example::
 
-            >>> client.delete_session('H5Qv-WhgSBGW0WL8xolSCQ', 'yL0dnd17RNe9Jpn7uAa8Bw')
+            >>> client.delete_session('H5Qv-WhgSBGW0WL8xolSCQ',
+            ...                       'yL0dnd17RNe9Jpn7uAa8Bw')
         """
 
-        url = '%(ep)s/%(version)s/%(tenant)s' \
-              '/users/%(user)s/sessions/%(session_id)s' % ({
-                  'ep': self.user_api_url, 'version': self.version,
-                  'tenant': self.tenant, 'user': user_id, 'session_id': session_id})
+        url = ('%(ep)s/%(version)s/%(tenant)s'
+               '/users/%(user)s/sessions/%(session_id)s') % {
+                   'ep': self.user_api_url, 'version': self.version,
+                   'tenant': self.tenant, 'user': user_id,
+                   'session_id': session_id
+               }
 
         res = self._perform_request('delete', url)
         return self._process_response(res, [204])
 
-    def get_users(self):
+    def get_users(self, include_config=None):
         """Returns all users within the logged-in users tenant.
 
+        :param include_config: Boolean Whether to include user configuration or
+            not. Defaults to True
+
         Example::
 
             >>> client.get_users()
@@ -287,13 +317,47 @@ class UserApiMixin(object):
                 }
             ]}
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/users' % ({
-            'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant})
+        params = {}
+        if include_config is not None:
+            params['include_config'] = include_config
 
-        res = self._perform_request('get', url)
+        url = '%(ep)s/%(version)s/%(tenant)s/users' % {
+            'ep': self.user_api_url, 'version': self.version,
+            'tenant': self.tenant
+        }
+
+        res = self._perform_request('get', url, params=params)
         return self._process_response(res, [200])
 
+    def add_user(self, email=None, password=None, role=None):
+        """
+        Add a new user to the current tenant.
+        
+        :param email: Email address of the user.
+        :param password: Password of the user.
+        :param role: role of the user in the tenant. Must be one of:
+            admin, user, demo, reader
+        """
+        headers = {'Content-Type': 'application/json'}
+        data = {}
+
+        email_regex = re.compile(r"[^@]+@[^@]+\.[^@]+")
+        if email and email_regex.match(email):
+            data['email'] = email
+        if password:
+            data['password'] = password
+        if role:
+            data['role'] = role
+
+        url = '%(ep)s/v0/%(tenant)s/users' % {
+            'ep': self.user_api_url, 'tenant': self.tenant
+        }
+        
+        res = self._perform_request('post', url,
+                                    data=json.dumps(data),
+                                    headers=headers)
+        return self._process_response(res, [201])
+
     def delete_user(self, user_id):
         """Delete a user from the system.
 
@@ -303,10 +367,10 @@ class UserApiMixin(object):
 
             >>> client.delete_user('H5Qv-WhgSBGW0WL8xolSCQ')
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/users/%(user_id)s' % ({
+        url = '%(ep)s/%(version)s/%(tenant)s/users/%(user_id)s' % {
             'ep': self.user_api_url, 'version': self.version,
             'tenant': self.tenant, 'user_id': user_id,
-        })
+        }
         res = self._perform_request('delete', url)
         return self._process_response(res, [204])
 
@@ -329,14 +393,12 @@ class UserApiMixin(object):
                u'permissions': [u'fingerprints.read.*',
                                 u'fingerprints.write.create.adhoc',
                                 u'items.read.*',
-                                u'objects.read.*',
                                 u'previews.read.*',
                                 u'projects.read.*',
                                 u'savedsearches.read.*',
                                 u'scores.read.*',
                                 u'signals.read.*',
-                                u'sources.read.*',
-                                u'subscriptions.read.*']},
+                                u'sources.read.*']},
                 …
             ]}
 
@@ -344,9 +406,11 @@ class UserApiMixin(object):
         has in this project. The permissions depend on the user's project role
         and server role.
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/projects/%(project_id)s/members' % ({
-            'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'project_id': project_id})
+        url = ('%(ep)s/%(version)s/%(tenant)s/'
+               'projects/%(project_id)s/members') % {
+                   'ep': self.user_api_url, 'version': self.version,
+                   'tenant': self.tenant, 'project_id': project_id
+               }
 
         res = self._perform_request('get', url)
         return self._process_response(res, [200])
@@ -355,10 +419,14 @@ class UserApiMixin(object):
                            group_id=None):
         """Adds a user or group to a project as a member.
 
-        :param project_id: Project identifier of the project in which to add the member.
-        :param role: Role of the member. Allowed values: `reader`, `member`, `admin`.
-        :param user_id: Identifier of the user to be added (can't be combined with `group_id`).
-        :param group_id: Identifier of the group to be added (can't be combined with `user_id`).
+        :param project_id: Project identifier of the project in which to add
+            the member.
+        :param role: Role of the member. Allowed values: `reader`, `member`,
+            `admin`.
+        :param user_id: Identifier of the user to be added (can't be combined
+            with `group_id`).
+        :param group_id: Identifier of the group to be added (can't be
+            combined with `user_id`).
 
         Example::
 
@@ -374,20 +442,20 @@ class UserApiMixin(object):
                u'project_id': '2sic33jZTi-ifflvQAVcfw'
                u'permissions': [u'fingerprints.*',
                                 u'items.*',
-                                u'objects.*',
                                 u'previews.*',
                                 u'projects.read.*',
                                 u'projects.write.*',
                                 u'savedsearches.*',
                                 u'scores.*',
                                 u'signals.*',
-                                u'sources.*',
-                                u'subscriptions.*']
+                                u'sources.*']
             }
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/projects/%(project_id)s/members' % ({
-            'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'project_id': project_id})
+        url = ('%(ep)s/%(version)s/%(tenant)s/'
+               'projects/%(project_id)s/members') % {
+                   'ep': self.user_api_url, 'version': self.version,
+                   'tenant': self.tenant, 'project_id': project_id
+               }
 
         data = {
             'user_id': user_id,
@@ -402,13 +470,17 @@ class UserApiMixin(object):
 
         Right now only the role can be changed, so that argument is required.
 
-        :param project_id: Project identifier of the project in which to modify the member.
+        :param project_id: Project identifier of the project in which to
+            modify the member.
         :param member_id: Identifier of the user or group to be updated.
-        :param role: New role of the member. Allowed values: `reader`, `member`, `admin`.
+        :param role: New role of the member. Allowed values: `reader`,
+            `member`, `admin`.
 
         Example::
 
-            >>> client.update_project_member('2sic33jZTi-ifflvQAVcfw', 'H5Qv-WhgSBGW0WL8xolSCQ', 'admin')
+            >>> client.update_project_member('2sic33jZTi-ifflvQAVcfw',
+            ...                              'H5Qv-WhgSBGW0WL8xolSCQ',
+            ...                              'admin')
             {
                u'email': u'user01@example.com',
                u'full_name': u'User 1',
@@ -417,18 +489,20 @@ class UserApiMixin(object):
                …
             }
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/projects/%(project_id)s/members/%(member_id)s' % ({
-            'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'project_id': project_id,
-            'member_id': member_id,
-        })
+        url = ('%(ep)s/%(version)s/%(tenant)s/'
+               'projects/%(project_id)s/members/%(member_id)s') % {
+                   'ep': self.user_api_url, 'version': self.version,
+                   'tenant': self.tenant, 'project_id': project_id,
+                   'member_id': member_id,
+               }
         res = self._perform_request('put', url, data={'role': role})
         return self._process_response(res)
 
     def delete_project_member(self, project_id, member_id):
         """Removes a user or group from a project as a member.
 
-        :param project_id: Project identifier of the project to remove the member from.
+        :param project_id: Project identifier of the project to remove the
+            member from.
         :param member_id: Identifier of the user or group to be removed.
 
         Example::
@@ -436,11 +510,12 @@ class UserApiMixin(object):
             >>> client.delete_project_member('2sic33jZTi-ifflvQAVcfw',
                                              'H5Qv-WhgSBGW0WL8xolSCQ')
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/projects/%(project_id)s/members/%(member_id)s' % ({
-            'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'project_id': project_id,
-            'member_id': member_id,
-        })
+        url = ('%(ep)s/%(version)s/%(tenant)s/'
+               'projects/%(project_id)s/members/%(member_id)s') % {
+                   'ep': self.user_api_url, 'version': self.version,
+                   'tenant': self.tenant, 'project_id': project_id,
+                   'member_id': member_id,
+               }
         res = self._perform_request('delete', url)
         return self._process_response(res, [204])
 
@@ -465,9 +540,9 @@ class UserApiMixin(object):
              }
             ]
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/groups' % ({
+        url = '%(ep)s/%(version)s/%(tenant)s/groups' % {
             'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant})
+            'tenant': self.tenant}
         res = self._perform_request('get', url)
         return self._process_response(res, [200])
 
@@ -485,17 +560,18 @@ class UserApiMixin(object):
                 u'members': []
             }
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/groups' % ({
+        url = '%(ep)s/%(version)s/%(tenant)s/groups' % {
             'ep': self.user_api_url, 'version': self.version,
             'tenant': self.tenant,
-        })
+        }
         res = self._perform_request('post', url, data={'name': name})
         return self._process_response(res, [201])
 
     def get_group(self, group_id):
         """Returns details, including the members, of a group.
 
-        :param group_id: Identifier of the group for which the details are returned.
+        :param group_id: Identifier of the group for which the details are
+            returned.
 
         Example::
 
@@ -515,9 +591,9 @@ class UserApiMixin(object):
                 ]
             }
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/groups/%(group)s' % ({
+        url = '%(ep)s/%(version)s/%(tenant)s/groups/%(group)s' % {
             'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'group': group_id})
+            'tenant': self.tenant, 'group': group_id}
         res = self._perform_request('get', url)
         return self._process_response(res, [200])
 
@@ -545,9 +621,9 @@ class UserApiMixin(object):
                 ]
             }
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/groups/%(group)s' % ({
+        url = '%(ep)s/%(version)s/%(tenant)s/groups/%(group)s' % {
             'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'group': group_id})
+            'tenant': self.tenant, 'group': group_id}
         res = self._perform_request('put', url, data={'name': name})
         return self._process_response(res, [200])
 
@@ -560,9 +636,10 @@ class UserApiMixin(object):
 
             >>> client.delete_group('1jo-WmmyRvC8UYtPMIQaoQ')
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/groups/%(group)s' % ({
+        url = '%(ep)s/%(version)s/%(tenant)s/groups/%(group)s' % {
             'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'group': group_id})
+            'tenant': self.tenant, 'group': group_id
+        }
         res = self._perform_request('delete', url)
         return self._process_response(res, [204])
 
@@ -585,9 +662,10 @@ class UserApiMixin(object):
                 u'tenant': 'lW8-FnZlQYWBimxFknYitw'
             }
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/groups/%(group)s' % ({
+        url = '%(ep)s/%(version)s/%(tenant)s/groups/%(group)s' % {
             'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'group': group_id})
+            'tenant': self.tenant, 'group': group_id
+        }
         res = self._perform_request('post', url, data={'user_id': user_id})
         return self._process_response(res, [200])
 
@@ -602,8 +680,10 @@ class UserApiMixin(object):
             >>> client.delete_group_member('1jo-WmmyRvC8UYtPMIQaoQ',
                                            'H5Qv-WhgSBGW0WL8xolSCQ')
         """
-        url = '%(ep)s/%(version)s/%(tenant)s/groups/%(group)s/members/%(user)s' % ({
-            'ep': self.user_api_url, 'version': self.version,
-            'tenant': self.tenant, 'group': group_id, 'user': user_id})
+        url = ('%(ep)s/%(version)s/%(tenant)s/'
+               'groups/%(group)s/members/%(user)s') % {
+                   'ep': self.user_api_url, 'version': self.version,
+                   'tenant': self.tenant, 'group': group_id, 'user': user_id
+               }
         res = self._perform_request('delete', url)
         return self._process_response(res, [204])
